
(* -------------------------------------------------------------------- *)
(*									*)
(*	Elab.ML:							*)
(*	Theorem prover for Elaboration					*)
(*					 				*)
(* -------------------------------------------------------------------- *)

structure ElabProver =
struct
local open elab_obj elab_obj_lemmas elab_program  in 

fun get_intr_rules (name, alist) =
  map (fn th => subst_vars th alist) 
    (mk_intr_rule (get_axiom thy name));


val intr_rules_data = 
  [ ("Scon", ["t"]),
    ("Var_", []),
    ("Con", []),
    ("ExCon", []),
    ("Empty_Record", ["t"]),
    ("Record", ["t"]),
    ("Let", []),
    ("Bracket", []),
    ("ExpRow1", ["r"]),
    ("ExpRow2", ["r'"]),
    ("Apply", []),
    ("Typed", []),
    ("Handle", []),
    ("Raise", []),
    ("Fn", []),
    ("Match_", []),
    ("Mrule", ["t"]),
    ("Val", ["E'"]),
    ("Type_", ["E"]),
    ("DataType", []),
    ("AbsType", []),
    ("Exception", ["E"]),
    ("Local", []),
    ("Nothing", ["E"]),
    ("Comp1", []),
    ("Comp2", []),
    ("ValBind1", []),
    ("ValBind2", []),
    ("Rec", []),
    ("TypBind1", ["TE"]),
    ("TypBind2", []),
    ("DatBind1", ["VE", "TE"]),
    ("DatBind2", []),
    ("ConBind1", ["CE"]),
    ("ConBind2", ["CE"]),
    ("ConBind3", []),
    ("ConBind4", []),
    ("ExBind1", ["EE"]),
    ("ExBind2", ["EE"]),
    ("ExBind3", []),
    ("ExBind4", []),
    ("ExBind5", ["EE'"]),
    ("ExBind6", []),
    ("Wild_Pat", ["VE"]),
    ("Scon_Pat", ["VE", "t"]),
    ("Var_Pat", ["VE"]),
    ("Con_Pat", ["VE'"]),
    ("ExCon_Pat", ["VE'"]),
    ("Empty_Record_Pat", ["VE", "t"]),
    ("Record_Pat", ["t"]),
    ("Bracket_Pat", []),
    ("Wild_PatRow", ["VE"]),
    ("PatRow1", ["r"]),
    ("PatRow2", ["VE''", "r'"]),
    ("Con_Val_Pat", []),
    ("ExCon_Val_Pat", []),
    ("Typed_Pat", []),
    ("Layered_Pat", []),
    ("Typed_Layered_Pat", []),
    ("TyVar_Ty'", ["'b"]),
    ("Empty_Record_Ty", ["t"]),
    ("Record_Ty", ["t"]),
    ("TyCon", []),
    ("Fn_Ty", ["t''"]),
    ("Bracket_Ty", []),
    ("TyRow1", ["r"]),
    ("TyRow2", ["r'"]),
    ("TySeq_Empty", ["typeseq"]),
    ("TySeq", ["typeseq'"]),
    ("Program1", []),
    ("Program2", [])];

val unflattened_intr_rules = 
     map get_intr_rules
         intr_rules_data;

val intr_rules = 
 flat unflattened_intr_rules;


(* -------------------------------------------------------------------- *)
(*	Some Derived Rules for the Initial Context			*)
(* -------------------------------------------------------------------- *)

fun tacfun prems = 
	     [cut_facts_tac prems 1,
              REPEAT (ares_tac 
               ([apply_VE0, apply_TE0, refl, subst_seq_empty_rule] @ 
                   intr_rules) 1)];

val Var_init = prove_goalw thy [C0_def]
 "[| <<Var, x>, s> : VE0; s [>] t |] ==> C0 |- var x ~> t" tacfun;

val Con_init = prove_goalw thy [C0_def]
 "[| <<Con, c>, s> : VE0; s [>] t |] ==> C0 |- con c ~> t" tacfun;

val ExCon_init = prove_goalw thy [C0_def]
 "<<ExCon, e>, t> : VE0 ==> C0 |- excon e ~> t" tacfun;

val Con_Pat_init = prove_goalw thy [C0_def]
 "[| <<Con, c>, s> : VE0; s [>] t |] ==> C0 ||- con c ~> (0, t) [Pat]" tacfun;

val ExCon_Pat_init = prove_goalw thy [C0_def]
 "<<ExCon, c>, exn> : VE0 ==> C0 ||- excon e ~> (0, exn) [Pat]" tacfun;

val ExBind5_init = prove_goalw thy [C0_def]
 "<<ExCon, e'>, t> : VE0 ==> C0 ||- excon e is e' ~> \
      \ {<<ExCon, e>, All, 0, t>} [ExBind]" tacfun;

val TySeq_empty_init = prove_goalw thy [C0_def]
 "<<TyCon, tycon>, <Lam, 0, t>, CE> : TE0 ==> C0 |- <> tycon ~> t [Ty]" tacfun;


val TySeq_init = prove_goalw thy [C0_def]
 "[| <<TyCon, tycon>, <Lam, tyvarseq, t'>, CE> : TE0; \
\    C0 |- tyseq ~> typeseq [TySeq] ; \
\    subst_seq (t', typeseq, tyvarseq, t) |] \
\       ==> C0 |- tyseq tycon ~> t [Ty]" tacfun;

(* ExBind6_init not done yet *)

(* -------------------------------------------------------------------- *)
(*	step tactics:	applies a language rule once, and safely	*)
(* -------------------------------------------------------------------- *)

val init_seq_rules = [Var_init, Con_init, ExCon_init, 
             Con_Pat_init, ExCon_Pat_init, 
             TySeq_empty_init, TySeq_init];

fun is_init_seq (_$ (S $ Const("_C0", _) $ phrase $ _)) = true |
    is_init_seq (_$ (S $ Const("_C0", _) $ phrase $ _ $ _ )) = true;


val elab_step_tac =   
 let val tac = ares_tac (VE0_mems @ TE0_mems) ORELSE' 
               resolve_tac intr_rules
 in
  SUBGOAL (fn (prem,i) => 
   (let val t = Logic.strip_assums_concl prem;
    in 
      if (is_init_seq t) then 
	 ares_tac init_seq_rules i ORELSE tac i
      else
  	 tac i 
    end
    handle _ => tac i))
  end;

(* -------------------------------------------------------------------- *)
(*	 elab_tac:	general-purpose tactic for proving goals 	*)
(*	 							 	*)
(*       Note: because the Rec rule (see ElabValBind.thy) allows us	*)
(*	 to infer the resulting type in a recursive declaration,  	*)
(*       some semantic goals must remain untouched until further 	*)
(*       instantiations have been made; hence the use of 		*)
(*       FIRSTGOAL o elab_step_tac					*)
(*       - contrast this with the step tactic for evaluation!! 		*)
(* -------------------------------------------------------------------- *)

val step_tac = (FIRSTGOAL elab_step_tac) ORELSE  obj_tac 1; 

val elaborate_tac  = REPEAT1 step_tac;

val set_tyvars_tac = Tactic (fn state =>  (marker := 1; 
                       tapply (TRY (PRIMITIVE set_tyvars), state)));

val elab_tac  = elaborate_tac THEN set_tyvars_tac; 

(* -------------------------------------------------------------------- *)
(*		 Interface Functions					*)
(* -------------------------------------------------------------------- *)

fun exp_elab phrase = 
  (prs "\n\n"; marker := 1; goal thy ("C0 |- "^phrase^" ~> ?t"));

fun dec_elab phrase = 
  (prs "\n\n"; marker := 1; goal thy ("C0 ||- "^phrase^" ~> ?E"));

fun program_elab phrase = 
  (prs "\n\n"; marker := 1; goal thy ("C0 ||- "^phrase^" ~> ?E [Program]"));

(* -------------------------------------------------------------------- *)
(*	 Pretty printing flags 						*)
(* -------------------------------------------------------------------- *)

val show_envs = ref false;
val show_contexts = ref false;
val show_full_types = ref false;


fun Envtr' [SE,TE,VE,EE] =  
  if (!show_envs) then raise Match else Const (" Env", dummyT); 

fun Contexttr' [T,U,E] =  
  if (!show_contexts) then raise Match else Const (" Context", dummyT); 

fun tr' [x] = 
  if (!show_full_types) then raise Match else x;

val thy = thy;

end
end;
