(* -------------------------------------------------------------------- *)
(*									*)
(*	Elle.ML: (Crude) User Interface Functions			*)
(*					 				*)
(* -------------------------------------------------------------------- *)

fun mk_program_seq I phrase = (I^" ||- "^phrase^" ~> ?X [Program]");

fun nothing () = ();

fun is_cr s = s = "\n";

fun is_comment s 
  = let val x = explode s; val y = rev x
    in  
       if (hd x = "(" andalso 
           hd (tl x) = "*" andalso 
           hd (tl y) = ")" andalso
           hd (tl (tl y)) = "*") then true else false
    end;

open PolyML.StringBuiltIns;

fun is_use s = substring (s,2,3)  = "use" handle _ => false;
 
fun interpret Context Env s lev =
let val flag1 = !SML_Syntax.show_full_syntax; (* user flag settings *)
    val flag2 = !eval_obj.show_closures;
    fun elab_get_VE (_ $ (_ $ (_ $ (_ $ (_ $ (_ $ VE) $ _))))) = VE;
    fun eval_get_VE           (_ $ (_ $ (_ $ VE) $ _)) = VE;
    val elab_th =  (* must have syntax in gory detail !! *)
                  (SML_Syntax.show_full_syntax := true;
                   prove_goal ElabProver.thy (mk_program_seq (!Context) s) 
                      (fn [] => [ElabProver.elab_tac]));
    val eval_th =  (* must have syntax in gory detail !! *)
                  (SML_Syntax.show_full_syntax := true;
                   eval_obj.show_closures := true;
                   prove_goal EvalProver.thy (mk_program_seq (!Env) s) 
                      (fn [] => [EvalProver.eval_tac]));
    val (_$(_$_$_$ context)) = concl_of elab_th;
    val (_$(_$_$_$ env)) = concl_of eval_th
in
    (Context :=  Sign.string_of_term (sign_of ElabProver.thy) context;
     Env :=  Sign.string_of_term (sign_of EvalProver.thy) env;
     SML_Syntax.show_full_syntax := flag1;
     eval_obj.show_closures := flag2;
     (if (lev = 0) then
                  (prs (Sign.string_of_term 
                       (sign_of EvalProver.thy) (eval_get_VE env));
                   prs " : \n  ";
                   writeln (Sign.string_of_term 
                       (sign_of ElabProver.thy) (elab_get_VE context)))
                      (* display theorems according to user flags !! *)
       else  
                  (writeln "Elaboration ...";
                   print_thm elab_th;
		   writeln "$ Evaluation ...";
                   print_thm eval_th)))
                      (* display theorems according to user flags !! *)
end
handle _ => writeln "Try again .. ";

fun REP () = 
  let val quit = ref false;
      val str = ref "";
      val Env = ref "E0";
      val Context = ref "C0";
      val level = ref 0;
      val infile = ref std_in
  in
     while (not (!quit)) do
       (prs "$ ";
        str := input_line (!infile);
        if (is_comment (!str)) then
            nothing()
        else if (is_use (!str)) then
            let val filename = substring (!str,6,size(!str)-6) in
                infile :=  open_in filename
            end 
        else case (!str) of
         "\n" => nothing() | 
         ":r\n" => (Env := "E0"; Context := "C0") | 
         ":q\n" => quit := true |
         ":lev 0\n" => level := 0 |
         ":lev 1\n" => level := 1 |
            _   =>  interpret Context Env (!str) (!level)) 
        handle _ => (prs "Enter :c to continue\n        :q to quit\n"; 
                     str := input_line std_in;
                     case (!str) of 
                       ":q\n" => quit := true |
                          _ => infile := std_in) 
  end; 
