(* -------------------------------------------------------------*)
(*	eval_obj.thy:  Semantic Objects for Evaluation		*)
(* ------------------------------------------------------------ *)

eval_obj = Kernel +

consts

 (* simple semantic objects *)
    
   "_Addr"	::	 "i"	("Addr")    (* addresses       *)
   "_ExName"	::	 "i"	("ExName")  (* exception names *)   
   "_BasVal"	::	 "i"	("BasVal")  (* basic values    *) 
   "_SVal"	::	 "i"	("SVal")    (* special values  *)   
   "_Fail"	::	 "i"	("Fail")    (* failure - singleton set *) 

    (* compound semantic objects  *)

   "_Val"	::	 "i"	("Val")     
   "_Val`"	::	 "i"	("Val`")     
   "_Record"	::	 "i"	("Record")     
   "_ExVal"	::	 "i"	("ExVal")     
   "_Pack"	::	 "i"	("Pack")     
   "_Mem"	::	 "i"	("Mem")     
   "_ExNameSet"	::	 "i"	("ExNameSet")     
   "_State"	::	 "i"	("State")     
   "_Closure"   ::       "i"    ("Closure")
   "_ConVal"	::	 "i"	("ConVal")
   "_Env"	::	 "i"	("Env")     
   "_Env`"	::	 "i"	("Env`")     
   "_StrEnv"	::	 "i"	("StrEnv")     
   "_VarEnv"	::	 "i"	("VarEnv")     
   "_ExConEnv"	::	 "i"	("ExConEnv")     
   "_Raised"	::	 "i"	("Raised")    (* for packets *) 

    (* initial environment and components *)

   "_E0"	::	 "i"	("E0")     
   "_SE0"	::	 "i"	("SE0")     
   "_VE0"	::	 "i"	("VE0")     
   "_EE0"	::	 "i"	("EE0")     
   " closure"	::	 "[Match, i] => i" ("closure '(_, ...')")

    (* semantic operations  *)

   env_combine	::	 "[i, i] => i"	    ("_ # _")     
   rec_		::	 "[i, i] => i"	    ("Rec '(_,_')")     
   FAIL		::	 "i"	            ("FAIL")(* for pattern matching *)    
   apply	::	 "[i, i] => i"	    ("apply '<_,_'>")     
   closure	::	 "[Match,i,i] => i" ("closure '(_,_,_')")     

   (* sequents for evaluation*)

   Exp_Seq	::     "[i,Exp,i] => o" ("(3_ |-/ _ ~> /_)")     
   ExpRow_Seq	::     "[i,ExpRow,i] => o" ("(3_ |- / _ ~> /_    [ExpRow])")
   Match_Seq	::     "[i,i,Match,i] => o" ("(3_ , _ |-/ _ ~> /_  [Match])")     
   Pat_Seq	::     "[i,i,Pat,i] => o" ("(3_ , _ ||-/ _ ~> /_  [Pat])")     
   PatRow_Seq	::     "[i,i,PatRow,i] => o" ("(3_ , _ ||-/ _ ~> /_  [PatRow])")     
   Dec_Seq	::     "[i,Dec,i] => o" ("(3_ ||-/ _ ~> /_)")     
   ValBind_Seq	::     "[i,ValBind,i] => o" ("(3_ ||-/ _ ~> /_  [ValBind])")     
   ExBind_Seq	::     "[i,ExBind,i] => o" ("(3_ ||-/ _ ~> /_  [ExBind])")     
   Program_Seq	::     "[i,Program,i] => o" ("(3_ ||-/ _ ~> /_  [Program])")     



   (* predefined objects *)

   MATCH	::	 "i"	("MATCH")     
   BIND		::	 "i"	("BIND")     
   " pr"	::	 "[i, i] => i" ("'(-_ , _-')")     

(* -------------------------------------------------------------*)
(*		Definitions of Semantic Objects			*)
(*	       (see page 47 of the Definition of SML)		*)
(* ------------------------------------------------------------ *)

rules 

Val_def 	  "Val == SUM x : \
\   {SVal, BasVal, Con, ConVal, ExVal, Record, Addr, Closure}. x"

Record_def	  "Record == Lab -fin-> Val"

ExVal_def	  "ExVal == SUM x : {ExName, ExName * Val}. x"

Pack_def	  "Pack == {Raised} * ExVal"

ConVal_def	  "ConVal == Con * Val"

Mem_def	 	  "Mem == Addr -fin-> Val"

ExNameSet_def	  "ExNameSet == Fin (ExName)"

State_def	  "State == Mem * ExNameSet"

Env_def		  "Env == StrEnv * VarEnv * ExConEnv"

StrEnv_def	  "StrEnv == StrId -fin-> Env"

VarEnv_def	  "VarEnv == (SUM x:{Var,Con,ExCon}.x) -fin-> Val"

ExConEnv_def	  "ExConEnv == ExCon -fin-> ExName"

    (*  Operations on Environments *)

env_combine_def	  "<SE,VE,EE> # <SE',VE',EE'> == <SE @@ SE',VE @@ VE',EE @@ EE'>"


    (*  Initial Environment  *)


E0_def        "E0 == <SE0, VE0, EE0>"

SE0_def       "SE0 == 0"

VE0_def       "VE0 == 						     \
\              {  <<Var, plus>,  <BasVal, plus>>,		     \
\	          <<Var, minus>, <BasVal, minus>>,  		     \
\                 <<Var, times>, <BasVal, times>>,  		     \
\                 <<Var, less>,  <BasVal, less>>,  		     \
\                 <<Var, zero>,  <BasVal, zero>>,  		     \
\                 <<Var, not>,   <BasVal, not>>,  		     \
\                 <<Var, head>,  <BasVal, head>>,  		     \
\                 <<Var, tail>,  <BasVal, tail>>,  		     \
\                 <<Var, null>,  <BasVal, null>>,  		     \
\                 <<Con, true>,  <Con, true>>,  		     \
\                 <<Con, false>, <Con, false>>,  		     \
\                 <<Con, nil>,   <Con, nil>>,  	 	    	     \
\                 <<Con, CONS>,  <Con, CONS>> }"

EE0_def        "EE0 == 0"

E0_type        "E0 : Env"

    (*  Rules expressing the fact that identifiers are distinct sets  *)

var_con_distinct	  "<Var, x> = <Con, y> ==> P"
var_excon_distinct	  "<Var, x> = <ExCon, y> ==> P"
con_excon_distinct	  "<Con, x> = <ExCon, y> ==> P"

   (* Predefined quantities *)


MATCH_ExName         "MATCH : ExName"

BIND_ExName          "BIND : ExName"

    (*  The apply function  *)


plus_rule   "(EX k. m #+ n = k & <SVal, k> = v) <-> apply <<BasVal,  plus>, \
\                   <Record, {<n1, SVal, m>, <n2, SVal, n>}>> = v"

minus_rule   "(EX k. m #- n = k & <SVal, k> = v) <-> apply <<BasVal,  minus>, \
\                   <Record, {<n1, SVal, m>, <n2, SVal, n>}>> = v"

times_rule   "(EX k. m #* n = k & <SVal, k> = v) <-> apply <<BasVal,  times>, \
\                   <Record, {<n1, SVal, m>, <n2, SVal, n>}>> = v"

zero_rule     "((<SVal, 0> = v & <Con, true> = v') | \
\               (<SVal, succ(n)> = v & <Con, false> = v'))  \
\                  <-> apply <<BasVal,  zero>, v> = v'"

not_rule      "((<Con, false> = v & <Con, true> = v') | \
\               (<Con, true> = v & <Con, false> = v')) \
\                   <-> apply <<BasVal,  not>, v> = v'"

head_rule      
 "v = v' <-> apply <<BasVal, head>,\
\      <Con * Val,CONS,Record,{<n1,v>, <n2,L>}>> = v'"

tail_rule      
 "L = L' <-> apply <<BasVal, tail>, \
\      <Con * Val,CONS,Record,{<n1,v>, <n2,L>}>> = L'"

null_rule     "((<Con, nil> = v & <Con, true>= v') | \
\               (<Con * Val,CONS,Record,{<n1,w>, <n2,L>}> = v \
\                               & <Con, false> = v'))  \
\                   <-> apply <<BasVal,  null>, v> = v'"

cons_rule      "<Con * Val,CONS,Record,{<n1,v>, <n2,L>}> = v' <-> \
\              apply <<Con, CONS>, (-v, L-)> = v'"


(* extension of apply *)


apply_con      "<Con * Val, c, v> = v' <-> apply <<Con, c>, v> = v'"

apply_excon    "<ExVal, ExName * Val, en, v> = v' <-> \
\         apply <<ExVal, ExName, en>, v> = v'"

apply_closure 
  "((EX VE' E''. Rec(VE, VE) = VE' & E' # <0, VE', 0> = E'' &        \
\     E'', v |- match ~> <Val, v'> [Match]) | \
\   (EX VE' E''. Rec(VE, VE) = VE' & E' # <0, VE', 0> = E'' &        \
\     E'', v |- match ~> FAIL [Match] & \
\       <Raised, ExVal, ExName, MATCH> = v')) <->       \
\                  apply <closure (match, E', VE), v> = v'"

Rec_empty_rule       "Rec (0, VE) = 0"
  
Rec_rule      "[| Rec(VE'', VE) = VE1;                              \
\    (cons (<<Var, f>, closure (match, E, VE)>, VE1) = VE''')  |] ==> \
\    Rec (cons (<<Var, f>, closure (match, E, VE')>, VE''), VE) = VE''' "

Rec_function       "VE' : A -> B ==> Rec (VE', VE) : A -> B"

Val_eq		"<Val, a> = <Val, b> <-> a = b"

Val_neq_pack    "~<Val, v> = <Raised, p>"

BasVal_neq_closure
	"~<BasVal, b> = closure(match,E,VE)"

ExVal_neq_closure
	"~<ExVal, b> = closure(match,E,VE)"

BasVal_neq_ExVal
	"~<BasVal, b> = <ExVal, e>"

closure_neq_empty	"~ closure (match, E, VE) = 0"

end 

ML

val show_closures = ref false;

fun closure_tr' [match, E, VE] =  
  if (!show_closures) then raise Match else Const (" closure", dummyT) $ match; 

val parse_translation = [] 
and  print_translation = [("closure", closure_tr')];

