
(* -------------------------------------------------------------*)
(*		More_ZF.ML: 					*)
(*		Further Derived Rules for ZF			*)
(* ------------------------------------------------------------ *)


val cs = ZF_cs addIs [equalityI] addEs [equalityE];

val tacfun = fn prems => [cut_facts_tac prems 1, fast_tac cs 1];

val empty_Un  =  prove_goal ZF.thy "A Un 0 = A" tacfun;

val Un_Diff1   =   prove_goal ZF.thy "A Int B = 0 ==> A Un B - A = B" tacfun;

val Un_Diff2   =   prove_goal ZF.thy "A Un A - A = 0" tacfun;

(* a more useful version of SigmaI *)

val Sigma1I = prove_goal  ZF.thy
    "[| a:A;  b:B(a); p = <a,b> |] ==> p : (SUM x:A. B(x))"
 (fn prems => [rewrite_goals_tac [Sigma_def],
               fast_tac (upair_cs addIs prems) 1]);

val apply_singleton = prove_goal ZF.thy
    "{<a,b>}`a = b"
   (fn prems => [rewrite_goals_tac [apply_def],
                 REPEAT (resolve_tac [the_equality, singletonI] 1 ORELSE
                         eresolve_tac [singletonE, Pair_inject2] 1)]);

val not_mem_Int = prove_goal ZF.thy 
 "[| A Int B = 0; x : A |] ==> ~x : B" tacfun;

val not_mem_cons = prove_goal ZF.thy 
 "cons (a, A) Int B = 0 ==> ~a : B" tacfun;

val not_eq_Int = prove_goal ZF.thy 
 "[| A Int B = 0; x : A; y : B; x = y |] ==> P" tacfun;

val domain_empty = prove_goal ZF.thy 
 "domain (0) = 0" tacfun;

val domain_cons = prove_goal ZF.thy 
 "domain (cons(<x,a>, f)) = cons (x, domain(f))" tacfun;

val pair_eq = prove_goal ZF.thy 
 "x = y ==> <A, x> = <A, y>" tacfun;

val not_sym = prove_goal ZF.thy 
 "~x = y ==> ~y = x" tacfun;


val cons_diff1 = prove_goal ZF.thy 
 "x:B ==> cons(x, A) - B = A - B"
 (fn prems => 
 [cut_facts_tac prems 1, 
  fast_tac (eq_cs addIs prems) 1]);


val cons_diff2 = prove_goal ZF.thy 
 "~x:B ==> cons(x, A) - B = cons(x, A - B)"
 (fn prems => 
 [cut_facts_tac prems 1, 
  fast_tac (eq_cs addIs prems) 1]);

val cons_un = prove_goal ZF.thy 
 "cons(x, A Un B) = cons (x,A) Un B"
(fn prems =>
  [cut_facts_tac prems 1,
   fast_tac (eq_cs addIs prems) 1]);

val restrict0 = prove_goalw ZF.thy [restrict_def] "restrict(f,0) = 0"
(fn prems => 
  [REPEAT (eresolve_tac [emptyE] 1 ORELSE
           resolve_tac [lam_type, fun_empty, fun_extension] 1)]);

val restrict1 = 
 prove_goalw ZF.thy [restrict_def] "f : Pi(A,B) ==> restrict(f,domain(f)) = f"
 (fn prems => 
  [cut_facts_tac prems 1,
   resolve_tac [eta] 1,
   asm_simp_tac (ZF_ss addsimps [domain_of_fun]) 1]);

val apply_equality1 = 
 prove_goal ZF.thy "[| f : A -> B; <a,b> : f |] ==> f`a = b"
  (fn prems => [cut_facts_tac prems 1, 
               fast_tac (cs addIs [apply_equality]) 1]);


val Un_cons_equality = prove_goal ZF.thy 
"[| B Un C = D; cons (a,D) = A |] ==> cons(a,B) Un C = A"
  (fn prems => [cut_facts_tac prems 1, 
                asm_simp_tac (ZF_ss addsimps [Un_cons]) 1]);


val Un_cons_equality_absorb =
prove_goal ZF.thy "[| a : B ; A Un B = C |] ==> cons(a,A) Un B = C"
(fn [p1, p2] => [cut_facts_tac [p1, p2] 1, 
                REPEAT (ares_tac [Un_cons_equality, cons_absorb] 1),
                simp_tac (ZF_ss addsimps [p2 RS sym]) 1, fast_tac ZF_cs 1]);

 
val if_true_equality = prove_goal ZF.thy "a = c ==> if(True,a,b) = c"
 (fn prems => [cut_facts_tac prems 1, 
               asm_simp_tac ZF_ss  1]);


val if_false_equality = prove_goal ZF.thy "b = c ==> if(False,a,b) = c"
 (fn prems => [cut_facts_tac prems 1, 
               asm_simp_tac ZF_ss  1]);
             








