(* -------------------------------------------------------------*)
(*		sml_syntax.thy					*)
(* ------------------------------------------------------------ *)

(* ------------------------------------------------------------ *)
(* 	NB: phrase classes are modelled by types in the logic 	*)
(*	    but identifiers are modelled by sets (see below) 	*)
(* ------------------------------------------------------------ *)

SML_Syntax = identifiers + 

types 
      ExpRow,    (* expression rows       *)
      Exp, 	 (* expressions           *)
      Match,     (* matches               *)
      Mrule,     (* match rules           *)
      Dec,       (* declarations          *)
      ValBind,   (* value bindings        *)
      TypBind,   (* type bindings         *)
      DatBind,   (* datatype bindings     *)
      ConBind,   (* constructor bindings  *)
      ExBind,    (* exception bindings    *)
      PatRow,    (* pattern rows          *)
      Pat,       (* patterns              *)
      Ty,        (* type expressions      *)
      TyRow,     (* type expression rows  *)
      Program,   (* programs              *)
      ExpSeq,    (* expression sequences  *)
      TySeq,     (* type sequences        *)
      TyArgs     (* type arguments        *) 0

arities
      ExpRow,    (* expression rows       *)
      Exp, 	 (* expressions           *)
      Match,     (* matches               *)
      Mrule,     (* match rules           *)
      Dec,       (* declarations          *)
      ValBind,   (* value bindings        *)
      TypBind,   (* type bindings         *)
      DatBind,   (* datatype bindings     *)
      ConBind,   (* constructor bindings  *)
      ExBind,    (* exception bindings    *)
      PatRow,    (* pattern rows          *)
      Pat,       (* patterns              *)
      Ty,        (* type expressions      *)
      TyRow,     (* type expression rows  *)
      Program,   (* programs              *)
      ExpSeq,    (* expression sequences  *)
      TySeq,     (* type sequences        *)
      TyArgs     (* type arguments        *) :: term


(* -------------------------------------------------------------------- *)
(*	    The Bare Language Syntax for the Core of SML		*)
(*	taken directly from the Definition of SML. pp 8-9		*)
(*	Note that it omits the following:				*)
(*		open declarations					*)
(*		fixity directives, and					*)
(*		"op" qualifiers						*)
(* -------------------------------------------------------------------- *)

consts

  (* priming type variables *)

   TyVar	::	 "i => i"    ("''_")   

  (* atexp *)

   Var_		::	 "i => Exp"            ("(var _)")   
   Scon		::	 "i => Exp"            ("(scon _)")   
   Con		::	 "i => Exp"            ("(con _)")   
   ExCon	::	 "i => Exp"            ("(excon _)")   
   Empty_Record	::	 "Exp"                 ("{{}}")   
   Record	::	 "ExpRow => Exp"       ("{{_}}")   
   Let		::	 "[Dec, Exp] => Exp"   ("(4let _ )/ (3in/ _) end")   
   Bracket	::	 "Exp => Exp"          ("('(- _ -'))") (* chicane ! *)

  (*  exprow *)

   ExpRow1	::	 "[i, Exp] => ExpRow"            ("_ = _")   
   ExpRow2	::	 "[i, Exp, ExpRow] => ExpRow"    ("_ = _ , _")   
  

  (*  exp *)

   Apply	::	 "[Exp, Exp] => Exp"        ("(1_ `/ _)")   
   Infix_Apply	::	 "[Exp, i, Exp] => Exp"     ("(1_  _/  _)")   
   Typed	::	 "[Exp, Ty] => Exp"         ("(1_ : _)")   
   Handle	::	 "[Exp, Match] => Exp"      ("(1_ handle _)")   
   Raise	::	 "Exp => Exp"               ("(1raise _)")   
   Fn		::	 "Match => Exp"             ("(1fn _)")   

  (*  match *)

   Mrule	::	 "[ Pat, Exp] =>  Match"        ("(_ =>/ _)")   
   Match_	::	 "[ Pat, Exp, Match] =>  Match" ("(_ =>/ _ |/ _)")   

  (* dec *)

   Val		::	 "ValBind => Dec"         ("(1val _)")   
   Type_	::	 "TypBind => Dec"         ("(1type _)")   
   DataType	::	 "DatBind => Dec"         ("(1datatype _)")  
   AbsType	::	 "[DatBind, Dec] => Dec"  ("(1abstype _ with _ end)")  
   Exception	::	 "ExBind => Dec"          ("(1exception _)")  
   Local	::	 "[Dec, Dec] => Dec"      ("(6local _)/ (6in/ _ end)")  
       (* NB:  'open' declaration omitted for now  *)
   Nothing	::	 "Dec"      ("nothing ")  
   Comp1	::	 "[Dec, Dec] => Dec"       ("(_/ _)")  
   Comp2	::	 "[Dec, Dec] => Dec"       ("(_;/ _)")  
       (* NB:   fixity directives omitted for now  *)

  (* valbind *)

   ValBind1	::	 "[Pat,  Exp] => ValBind"     ("(1_ =/ _)")  
   ValBind2	::	 "[Pat,  Exp, ValBind] => ValBind" ("(1_ =/ _ and/ _)")  
   Rec		::	 "ValBind => ValBind"     ("(1rec _ )")  


  (* typbind *)

   TypBind1	::	 "[i, i, Ty] => TypBind"     ("_ _ is _")  
   TypBind2	::	 "[i, i, Ty, TypBind] => TypBind"   ("_ _ is _ and _")  

  (* datbind *)

   DatBind1	::	 "[i, i, ConBind] => DatBind"   ("_ _ = _") 
   DatBind2	::	 "[i, i, ConBind, DatBind] => DatBind"  ("_ _ = _ and _") 

  (* conbind *)

   ConBind1	::	"i => ConBind"  ("con _") 
   ConBind2	::	"[i, Ty] => ConBind"  ("con _ of _") 
   ConBind3	::	"[i, ConBind] => ConBind"   ("con _ | _") 
   ConBind4	::	"[i, Ty, ConBind] => ConBind" ("con _ of _ | _") 

  (* exbind *)

   ExBind1	::	"i => ExBind"                ("excon _") 
   ExBind2	::	"[i, Ty] => ExBind"          ("excon _ of _")
   ExBind3	::	"[i, ExBind] => ExBind"      ("excon _ and _")
   ExBind4	::	"[i, Ty, ExBind] => ExBind"  ("excon _ of _ and _")
   ExBind5	::	"[i, i] => ExBind"           ("excon _ is _") 
   ExBind6	::	"[i, i, ExBind] => ExBind"   ("excon _ is _ and _")

  (* atpat *)

   Wild_Pat	::	"Pat"           ("'_") 
   Scon_Pat	::	"i => Pat"      ("scon _") 
   Var_Pat	::	"i => Pat"      ("var _") 
   Con_Pat	::	"i => Pat"      ("con _") 
   ExCon_Pat	::	"i => Pat"      ("excon _") 
   Empty_Record_Pat ::	"Pat"           ("{{}}") 
   Record_Pat	::	"PatRow => Pat" ("{{_}}") 
   Bracket_Pat	::	"Pat => Pat"    ("('(- _ -'))") 

  (* patrow *)

   Wild_PatRow	::	"PatRow"                  ("...") 
   PatRow1	::	"[i, Pat] => PatRow"      ("_ = _") 
   PatRow2	::	"[i, Pat, PatRow] => PatRow"      ("_ = _, _") 


  (* pat *)

   Con_Val_Pat		::	"[i, Pat] => Pat"       ("con _`_") 
   ExCon_Val_Pat	::	"[i, Pat] => Pat"       ("excon _`_") 
   Infix_Con_Val_Pat	::	"[Pat, i, Pat] => Pat"  ("_ con _ _") 
   Infix_ExCon_Val_Pat	::	"[Pat, i, Pat] => Pat"  ("_ excon _ _")
   Typed_Pat		::	"[Pat, Ty] => Pat"      ("_ : _") 
   Layered_Pat		::	"[i, Pat] => Pat"       ("var _ as _") 
   Typed_Layered_Pat	::	"[i, Ty, Pat] => Pat"   ("var _ : _ as _")

  (* ty *)

   TyVar_Ty	::	"i =>  Ty"            ("%_") 
   Empty_Record_Ty ::	"Ty"                  ("{{}}") 
   Record_Ty	::	"TyRow =>  Ty"        ("{{_}}") 
   TyCon	::	"[TySeq, i]  =>  Ty"  ("_ _") 
   Fn_Ty	::	"[Ty, Ty] => Ty"      ("_ -ty-> _")
   Bracket_Ty	::	"Ty => Ty"            ("'(- _ -')")


  (* tyrow *)

   TyRow1	::	"[i, Ty] =>  TyRow"        ("_ = _") 
   TyRow2	::	"[i, Ty, TyRow] => TyRow"  ("_ = _ , _")


  (* program *)
      
   Program1	::	"Dec => Program"        ("_ ;") 
   Program2	::	"[Dec, Program] => Program" ("_ ; _") 

(* -------------------------------------------------------------------- *)
(*		 Derived Forms	 					*)
(*	   (see parse and print translations below)			*)
(* -------------------------------------------------------------------- *)
 
  (* exp *)

   " Pr"	::	 "[Exp,  Exp] => Exp"           ("(1'(-_,/ _-'))")  
   " Plus"	::	 "[Exp,  Exp] => Exp"           ("(1_ ++/ _)")  
   " Minus"	::	 "[Exp,  Exp] => Exp"           ("(1_ --/ _)")  
   " Times"	::	 "[Exp,  Exp] => Exp"           ("(1_ **/ _)")  
   " Less"	::	 "[Exp,  Exp] => Exp"           ("(1_ </ _)")  
   " Orelse"	::	 "[Exp,  Exp] => Exp"           ("(1_ orelse/ _)")  
   " Andalso"	::	 "[Exp,  Exp] => Exp"           ("(1_ andalso/ _)")  
   " If"	::	 "[Exp,  Exp, Exp] => Exp"   ("(1if _/ then _/ else _)")  
   " Case"	::	 "[Exp,  Match] => Exp"         ("(1case _/ of _)")  
   " Zero"	::	 "Exp => Exp"                   ("(1zero _)") 
   " Knot"	::	 "Exp => Exp"                   ("(1not _)")  
   " Nil"	::	 "Exp"         			("[]")  
   " Cons"	::	 "[Exp,  Exp] => Exp"           ("(1_ ::/ _)")  
   " List"	::	 "ExpSeq =>  Exp"               ("[_]")  
   " Head"	::	 "Exp => Exp"                   ("(1head _)")  
   " Tail"	::	 "Exp => Exp"                   ("(1tail _)")  
   " Null"	::	 "Exp => Exp"                   ("(1null _)")  

  (* pat *)

   " Pr_Pat"	::	 "[Pat,  Pat] => Pat"           ("(1'(-_,/ _-'))")  

  (* ty *)

   " Pr_Type"	::	 "[Ty,  Ty] => Ty"              ("_ <*> _") 
   " Int"	::	 "Ty"                           ("Int")  
   " Bool"	::	 "Ty"                           ("Bool")  
   " Exn"	::	 "Ty"                           ("Exn")  

  (* tyseq *)

   " tyempty"	::	 "TySeq"                        ("<>")  
   " TyList"	::	 "TyArgs =>  TySeq"             ("<_>")  


  (* program *)

   " Exp_Program"	::	 "Exp =>  Program"       ("_;")  

(* -------------------------------------------------------------------- *)
(* 	Sequences are put last so that they are not 			*)
(*	confused with other syntactic forms involving 			*)
(*		comma separated items 					*)
(* -------------------------------------------------------------------- *)

   " ExpSeq1"	::	 "Exp => ExpSeq"                ("_")  
   " ExpSeq2"	::	 "[Exp, ExpSeq] =>  ExpSeq"     ("_,_")  
   "TySeq_Empty"::	 "TySeq"                        ("tyempty")  (*needed?*) 
   "TySeq"	::	 "[Ty,  TySeq] =>  TySeq"       ("_ tycons _")  
   " TyArgs1"	::	 "Ty => TyArgs"                 ("_")  
   " TyArgs2"	::	 "[Ty, TyArgs] => TyArgs"       ("_, _") 

end

ML

(* -------------------------------------------------------------------- *)
(*			 Parse  Translations  				*)
(* -------------------------------------------------------------------- *)

fun mk_tr name injection = 
  fn [] => mk_xn injection (pos name (mk_ide_list injection));

(* -------------------------------------------------------------------- *)
(*	 		 Derived Forms 					*)
(* -------------------------------------------------------------------- *)

fun unoptr name [t] = Const("Apply", dummyT) $ 
		   (Const ("Var_", dummyT) $ mk_tr name "invar" []) $ t;

fun Prtr [s, t] = 
    Const ("Record",dummyT) $ (Const ("ExpRow2",dummyT) $
     mk_tr "n1" "inlabel" [] $ s  
        $ (Const ("ExpRow1",dummyT) $  mk_tr "n2" "inlabel" [] $  t));;

fun binoptr name [s,t] = 
   Const("Apply", dummyT) $  
     (Const ("Var_", dummyT) $ mk_tr name "invar" []) $ Prtr [s,t];
 
val Zerotr   = unoptr "zero";
val Knottr   = unoptr "not";
val Nulltr   = unoptr "null";
val Headtr   = unoptr "head";
val Tailtr   = unoptr "tail";
val Plustr   = binoptr "plus";
val Minustr  = binoptr "minus";
val Timestr  = binoptr "times";
val Lesstr   = binoptr "less";

(* lists require some machinery *)

fun Niltr [] = Const("Con", dummyT) $ mk_tr "nil" "incon" [];

fun  Constr [s, t] = 
   Const("Apply", dummyT) $ (Const ("Con", dummyT) $ 
    mk_tr "CONS" "incon" []) $ Prtr [s,t];

fun Listtr [Const(" ExpSeq1", _) $ s] = Constr [s, Niltr []] |
    Listtr [Const(" ExpSeq2", _) $ s $ t] = Constr [s, Listtr [t]] ;

fun Casetr [t1,t2] =  Const ("Apply", dummyT) $ 
     (Const("Fn", dummyT) $ t2) $ 
       (Const("Bracket", dummyT) $ t1); (* force argument to be atomic *)

fun Iftr [t1,t2,t3] = Casetr [t1,  Const("Match_", dummyT) $ 
     (Const("Con_Pat", dummyT) $ mk_tr "true" "incon" []) $
      t2 $ (Const("Mrule", dummyT) $
	   (Const("Con_Pat", dummyT) $ mk_tr "false" "incon" []) $ t3)];

fun Orelsetr [t1,t2] = Iftr [t1, 
     Const("Con", dummyT) $ mk_tr "true" "incon" [], t2];

fun Andalsotr [t1,t2] = Iftr [t1,t2, 
     Const("Con", dummyT) $ mk_tr "false" "incon" []];

fun Exp_Programtr [t] = Const("Program1", dummyT) $ 
   (Const("Val", dummyT) $ (Const ("ValBind1", dummyT) $ 
     (Const("Var_Pat", dummyT) $ mk_tr "it" "invar" []) $ t));

fun Pr_Pattr [s, t] = 
    Const ("Record_Pat",dummyT) $ (Const ("PatRow2",dummyT) $
      mk_tr "n1" "inlabel" [] $ s  
        $ (Const ("PatRow1",dummyT) $ mk_tr "n2" "inlabel" [] $  t));

fun Pr_Typetr [s, t] = 
    Const ("Record_Ty",dummyT) $ (Const ("TyRow2",dummyT) $
      mk_tr "n1" "inlabel" [] $ s  
        $ (Const ("TyRow1",dummyT) $ mk_tr "n2" "inlabel" [] $  t));

(* type constructions *)

fun Inttr [] = Const ("TyCon", dummyT) $ 
     Const("TySeq_Empty", dummyT) $ Const("int_con", dummyT);

fun Booltr [] = Const ("TyCon", dummyT) $  
     Const("TySeq_Empty", dummyT) $ Const("bool_con", dummyT);

fun Unittr [] = Const ("TyCon", dummyT) $  
     Const("TySeq_Empty", dummyT) $ Const("unit_con", dummyT);

fun Exntr [] = Const ("TyCon", dummyT) $  
     Const("TySeq_Empty", dummyT) $ Const("exn_con", dummyT);

fun tyemptytr [] = Const ("TySeq_Empty", dummyT);

fun TyListtr [Const(" TyArgs1", _) $ s] 
 = Const("TySeq", dummyT) $ s $ tyemptytr [] |
    TyListtr [Const(" TyArgs2", _) $ s $ t] 
 = Const("TySeq", dummyT) $ s $ TyListtr [t] ;

(* -------------------------------------------------------------------- *)
(*			 Print  Translations  				*)
(* -------------------------------------------------------------------- *)

(* -------------------------------------------------------------------- *)
(*     			Derived Forms				 	*)
(* -------------------------------------------------------------------- *)


fun stringtr " zero"  = " Zero" |
    stringtr " not"   = " Knot" |
    stringtr " null"  = " Null" |
    stringtr " head"  = " Head" |
    stringtr " tail"  = " Tail" |
    stringtr " plus"  = " Plus" |
    stringtr " minus" = " Minus" |
    stringtr " times" = " Times" |
    stringtr " less" = " Less" |
    stringtr " nil" = " Nil" ;


fun Contr' [Const("incon", _) $  w] = 
   let val Const(name, _) = mk_tr' "incon" [w] 
   in   
      Const(stringtr name, dummyT)
   end  |
   Contr' [s] = s;


fun Recordtr' 
 [ Const ("ExpRow2",_) $
    (Const("inlabel",_) $ (Const("black", _) $ Const("0",_))) $ s  $ 
     (Const ("ExpRow1",_) $
    (Const("inlabel",_) $ (Const("white", _) $ Const("0",_))) $  t)]
     =  Const(" Pr", dummyT) $ s $ t;

fun Record_Pattr' 
 [ Const ("PatRow2",_) $
    (Const("inlabel",_) $ (Const("black", _) $ Const("0",_))) $ s  $ 
     (Const ("PatRow1",_) $
    (Const("inlabel",_) $ (Const("white", _) $ Const("0",_))) $  t)]
     =  Const(" Pr_Pat", dummyT) $ s $ t;

fun Record_Tytr' 
 [ Const ("TyRow2",_) $
    (Const("inlabel",_) $ (Const("black", _) $ Const("0",_))) $ s  $ 
     (Const ("TyRow1",_) $
    (Const("inlabel",_) $ (Const("white", _) $ Const("0",_))) $  t)]
     =  Const(" Pr_Type", dummyT) $ s $ t;

fun mk_seq 
      (Const("Apply", _) $ 
      (Const ("Con", _) $ (Const("incon", _) $ (Const("black", _) $ 
           (Const("white", _) $ Const("0", _))))) $
      (Const ("Record",_) $ 
        (Const ("ExpRow2",_) $ 
         (Const("inlabel",_) $ (Const("black", _) $ Const("0",_))) $ s  $
        (Const ("ExpRow1",_) $ 
          (Const("inlabel",_) $ (Const("white", _) $ Const("0",_))) $
	       (Const ("Con", _) $ (Const("incon", _) $ (Const("black", _) $ 
           (Const("black", _) $ Const("0", _)))))))))
      =  Const(" ExpSeq1", dummyT) $ s |
    mk_seq 
     (Const("Apply", _) $
     (Const ("Con", _) $ (Const("incon", _) $ (Const("black", _) $ 
           (Const("white", _) $ Const("0", _))))) $
     (Const ("Record",_) $ 
        (Const ("ExpRow2",_) $ 
          (Const("inlabel",_) $ (Const("black", _) $ Const("0",_))) $ s  $
        (Const ("ExpRow1",_) $ 
           (Const("inlabel",_) $ (Const("white", _) $ Const("0",_))) $ t))))
      =  Const(" ExpSeq2", dummyT) $ s $ mk_seq t;

fun  Applytr'  
 [Const ("Con", _) $ (Const("incon", _) $ (Const("black", _) $ 
           (Const("white", _) $ Const("0", _)))) ,
  Const ("Record",_) $ 
     (Const ("ExpRow2",_) $ 
       (Const("inlabel",_) $ (Const("black", _) $ Const("0",_))) $ s  $ 
     (Const ("ExpRow1",_) $ 
       (Const("inlabel",_) $ (Const("white", _) $ Const("0",_))) $ t))]
     = (Const (" List", dummyT) $ 
      mk_seq  (Const("Apply", dummyT) $ 
              (Const ("Con", dummyT) $ 
	     (Const("incon", dummyT) $ (Const("black", dummyT) $ 
           (Const("white", dummyT) $ Const("0", dummyT))))) $
              (Const ("Record",dummyT) $ 
                 (Const ("ExpRow2",dummyT) $ 
                  (Const("inlabel", dummyT) $ 
                   (Const("black", dummyT) $ Const("0",dummyT))) $ s  $
                 (Const ("ExpRow1",dummyT) $  
                   (Const("inlabel",dummyT) $ 
                     (Const("white", dummyT) $ Const("0",dummyT))) $ t))))
    handle _ => Const (" Cons", dummyT) $ s $ t) |
  
  Applytr'
   [ Const("Fn", _) $ 
     (Const("Match_", _) $ 
      (Const("Con_Pat", _) $ 
       (Const("incon", _) $ (Const("black", _) $ Const("0", _)))) $
       (Const("incon", _) $ (Const("black", _) $ Const("0", _))) $ 
       (Const("Mrule", _) $ 
        (Const("Con_Pat", _) $ 
          (Const("incon", _) $ (Const("white", _) $ Const("0", _)))) $ t2)) , 
     Const("Bracket", dummyT) $ t1]
	   
         = Const(" Orelse", dummyT) $ t1 $ t2 |
    
  Applytr' 
   [ Const("Fn", _) $ 
     (Const("Match_", _) $ 
      (Const("Con_Pat", _) $ 
         (Const("incon", _) $ (Const("black", _) $ Const("0", _)))) $ t2 $ 
      (Const("Mrule", _) $ 
          (Const("Con_Pat", _) $ 
            (Const("incon", _) $ 
              (Const("white", _) $ Const("0", _)))) $
            (Const("incon", _) $ (Const("white", _) $ Const("0", _))))) , 
     Const("Bracket", dummyT) $ t1]
	 
         = Const(" Andalso", dummyT) $ t1 $ t2 |

   Applytr'
    [ Const("Fn", _) $ 
      (Const("Match_", _) $ 
       (Const("Con_Pat", _) $ 
        (Const("incon", _) $ (Const("black", _) $ Const("0", _)))) $  t2 $ 
       (Const("Mrule", _) $
           (Const("Con_Pat", _) $
             (Const("incon", _) $ (Const("white", _) $ Const("0", _)))) $ t3)) , 
      Const("Bracket", dummyT) $ t1]

	   = Const (" If", dummyT) $ t1 $ t2 $ t3 |

   Applytr' 
    [ Const("Fn", _) $ t2 , Const("Bracket", dummyT) $ t1]
 
           = Const(" Case", dummyT) $ t1 $ t2 |

   Applytr' 
    [ Const ("Var_", _)  $ (Const("invar", _) $  w),
      Const ("Record",_) $ 
       (Const ("ExpRow2",_) $ 
         (Const("inlabel",_) $ (Const("black", _) $ Const("0",_))) $ s  $ 
       (Const ("ExpRow1",_) $ 
         (Const("inlabel",_) $ (Const("white", _) $ Const("0",_))) $  t))]

       = let val Const(name, _) = mk_tr' "invar" [w] 
         in 
            Const(stringtr name, dummyT) $ s $ t
         end |

   Applytr'  [Const ("Var_", _) $ (Const("invar", _) $  w), t] 

       = let val Const(name, _) = mk_tr' "invar" [w] 
         in 
            Const(stringtr name, dummyT) $ t
         end;

fun TySeq_Emptytr' [] = Const(" tyempty", dummyT);


fun TyContr' [Const("TySeq_Empty", _), Const("int_con", _)] = 
      Const(" Int", dummyT) |
    TyContr' [Const("TySeq_Empty", _), Const("bool_con", _)] =          
      Const(" Bool", dummyT) |
    TyContr' [Const("TySeq_Empty", _), Const("unit_con", _)] =          
      Const(" Unit", dummyT) |
    TyContr' [Const("TySeq_Empty", _), Const("exn_con", _)] =          
      Const(" Exn", dummyT);

local fun f (Const("TySeq", _) $ s $ Const("TySeq_Empty", _)) 
  = Const(" TyArgs1", dummyT) $ s |
          f (Const("TySeq", _) $ s $ t)
  =  Const(" TyArgs2", dummyT) $ s $ (f t)
in 
    fun TySeqtr' [s, t] = 
    Const(" TyList", dummyT) $ f (Const("TySeq", dummyT) $ s $ t)
end;


val show_full_syntax = ref true;

fun tr' [x] = 
  if (!show_full_syntax) then raise Match else x;

(* -------------------------------------------------------------------- *)
(*	 Isabelle Syntax for SML 					*)
(* -------------------------------------------------------------------- *)

val parse_translation =
                             [(" Plus", Plustr), 
                              (" Minus", Minustr),
                              (" Times", Timestr) , 
                              (" Less", Lesstr) , 
                              (" Pr", Prtr) , 
                              (" Case", Casetr) , 
                              (" Orelse", Orelsetr) , 
                              (" Andalso", Andalsotr) , 
                              (" If", Iftr) , 
                              (" Exp_Program", Exp_Programtr) , 
                              (" Knot", Knottr), 
                              (" Zero", Zerotr), 
                              (" Null", Nulltr), 
                              (" Head", Headtr), 
                              (" Tail", Tailtr), 
                              (" Pr_Pat", Pr_Pattr) , 
                              (" Pr_Type", Pr_Typetr) , 
                              (" Nil", Niltr) , 
                              (" Cons", Constr),
                              (" List", Listtr),
                              (" Int", Inttr),
                              (" Bool", Booltr),
                              (" Exn", Exntr),
                              (" tyempty", tyemptytr),
                              (" TyList", TyListtr)]	  
and  print_translation =
                             [("Con", Contr'),
			      ("Apply", Applytr'),
                              ("Record", Recordtr'),
                              ("Record_Pat", Record_Pattr'),
                              ("Record_Ty", Record_Tytr'),
			      ("TySeq_Empty", TySeq_Emptytr'),
                              ("TyCon", TyContr'),
                              ("TySeq", TySeqtr'),
                              ("Scon", tr'),
                              ("Var_", tr'),
                              ("ExCon", tr'),
                              ("Scon_Pat", tr'),
                              ("Var_Pat", tr'),
                              ("Con_Pat", tr'),
                              ("ExCon_Pat", tr')];


